/*********************************************************************************
*  Seg 2501, hiver 2002 Devoir # 4
*
*  fichier Banque.java
*  contrlleur du simulateur et
*  calcule des statistiques
*
*  @author  Anne Bertrand
*  @author  Robin Tropper
*  @refactoredBy Robin Tropper  B.Sc.A.
*  
*  BOGUE : il arrivait (pass) parfois que les caissires ne voyaient pas que
*  la banque "est ferme" => rsultat: boucle infinie lorsque la caissire cherche
*  le prochain client.
*  => DEVRAIT tre rgl, mais difficile  prouver formellement. 
*  => DPANNAGE : si a se produit, simplement arrter et redmarrer.
*
**********************************************************************************/

import java.util.*;

public class Banque {

 	private static int nombreCaisses = 3;
 	private static long duree = 3600;
 	
 	private static boolean heureDeCloture = false;

 	static Timer fini = new Timer();	//temps limite pour l'excution du programme entier
	static FileClient fileEntr = new FileClient(300,100,duree, " Entreprise");
	static FileClient fileIndiv = new FileClient(50,50,duree," Individu");

	/**
	 * Par dfaut, il n'y a qu'une file pour entreprises et une file
	 * pour individus. Cette mthode a pour but de favoriser l'extensibilit
	 * dans le cas o multiples files  multiples priorits viendraient.
	 * 
	 * @return : la liste des files d'attente en ordre descendant de priorit.
	 * En d'autres mots, parcourir la liste assure le respect des priorits.
	 */
	public static synchronized List<FileClient> montreFiles(){
		List<FileClient> listeFiles = new ArrayList<FileClient>();
		listeFiles.add(0, fileEntr);
		listeFiles.add(1, fileIndiv);
		return listeFiles;
	}//fin montreFiles
	
	public static boolean estFerme(){
		return heureDeCloture;
	}//fin getCloture
	
//#####
// main
//#####
	public static void main(String[] args) {
		System.out.println("Simulation pour "+nombreCaisses+" caissi�res sur "+
								duree/600+" heures.");
		System.out.println("Veuillez patienter "+duree/600+
											" secondes pour les r�sultats");
		fini.schedule(new Terminer(), duree);
		System.out.println("NOMBRE CAISSES "+nombreCaisses);
		for (int k=0; k<nombreCaisses; k++) {
			new Caissiere(k);
		}
	}//fin du main

	static class Terminer extends TimerTask{
  		public void run(){
  			heureDeCloture = true;
  			fini.cancel();//arrter tous les autres objets qui l'utilisent

  			long EntAtt = fileEntr.SizeFile();			//Entreprises non-servis
	  		long IndAtt = fileIndiv.SizeFile();			//Individus non-servis
			long entrServis = fileEntr.getClientsServis();	//Entreprises servis
			long indivServis = fileIndiv.getClientsServis();	//Individus servis
			long entrVenus = EntAtt+entrServis;
			long indivVenus = IndAtt+indivServis;
			long servisTotal = entrServis+indivServis;
			long clientsTotal = servisTotal+EntAtt+IndAtt;
			long EntMoyAtt=0;
			long IndMoyAtt=0;
	  		System.out.println("\n"+clientsTotal+"Visiteurs au total.");
	  		System.out.println(nombreCaisses+" Caisses pour les servir.\n");
	  		System.out.println(entrVenus+" entreprises se sont prsentes.");
	  		System.out.println(indivVenus+" individus se sont prsentes.");
	  		System.out.println("Entreprises non-servis :" + EntAtt);
	  		System.out.println("Individus non-servis :" + IndAtt);
			System.out.println("\nClients total servis: "+clientsTotal);
			System.out.println("Entreprises servis: "+entrServis);
			System.out.println("Individus servis: "+indivServis);
			if ((EntMoyAtt=fileEntr.getMoyAttente())!=0) {
				System.out.println("\nMoyenne d'attente pour les entreprises "+
										EntMoyAtt/10+" minutes");
			} else { System.out.println("Aucune Entreprise servie.");}
			if ((IndMoyAtt=fileIndiv.getMoyAttente())!=0) {
				System.out.println("Moyenne d'attente pour les individus "+
										IndMoyAtt/10+" minutes\n");
			} else { System.out.println("Aucun Individu servi.");}
	  		System.exit(0);
  		}//fin du run
  	}//fin du TimerTask

}//fin de la classe Banque